--Klein, keyschedule

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

ENTITY klein_keyschedule IS
  PORT (
    key_in  : IN STD_LOGIC_VECTOR(79 DOWNTO 0);
    i       : IN STD_LOGIC_VECTOR(4 DOWNTO 0);
    key_out : OUT STD_LOGIC_VECTOR(79 DOWNTO 0)
    );
END klein_keyschedule;

ARCHITECTURE rtl OF klein_keyschedule IS

SIGNAL a : STD_LOGIC_VECTOR(39 DOWNTO 0); --for the 1st step of the
                                                 --keysch. (rotation)
SIGNAL b : STD_LOGIC_VECTOR(39 DOWNTO 0); --for the 2nd step of the
                                          --keysch.
SIGNAL c : STD_LOGIC_VECTOR(39 DOWNTO 0);

signal tmp : STD_LOGIC_VECTOR(4 DOWNTO 0);

COMPONENT klein_sbox IS
  PORT (
    a_in  : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    a_out : OUT STD_LOGIC_VECTOR(3 DOWNTO 0)
    );
END COMPONENT;


BEGIN

  --"rotation"
  b(31 DOWNTO 0)  <= key_in(79 DOWNTO 48);
  b(39 DOWNTO 32) <= key_in(47 DOWNTO 40);

  a(31 DOWNTO 0)  <= key_in(39 DOWNTO 8);
  a(39 DOWNTO 32) <= key_in(7 DOWNTO 0);
  
  tmp <= (b(16) & b(23 DOWNTO 20)) XOR STD_LOGIC_VECTOR(UNSIGNED(i)+1);

  --2nd step
  key_out(19 DOWNTO 0)  <= b(19 DOWNTO 17) & tmp(4) & b(15 downto 0);
  key_out(23 DOWNTO 20) <= tmp(3 downto 0);
  key_out(39 DOWNTO 24) <= b(39 DOWNTO 24);

  c <= a XOR b;

  key_out(47 DOWNTO 40) <= c(7 DOWNTO 0);
  key_out(79 DOWNTO 64) <= c(39 DOWNTO 24);
  

  i_klein_sbox1 : klein_sbox
    PORT MAP (
      a_in  => c(11 DOWNTO 8),
      a_out => key_out(51 DOWNTO 48)
      );

  i_klein_sbox2 : klein_sbox
    PORT MAP (
      a_in  => c(15 DOWNTO 12),
      a_out => key_out(55 DOWNTO 52)
      );
  
  i_klein_sbox3 : klein_sbox
    PORT MAP (
      a_in  => c(19 DOWNTO 16),
      a_out => key_out(59 DOWNTO 56)
      );

   i_klein_sbox4 : klein_sbox
    PORT MAP (
      a_in  => c(23 DOWNTO 20),
      a_out => key_out(63 DOWNTO 60)
      );
  
END rtl;

